; Warren-Hanson algorithm for generating 
; permutations of multisets.
; 
(define (make-k-permutations k multiset)
(let ((pivots (unique multiset)))
  (if (= k 1)
    (map list pivots)
    (let ((acc '()))
      (dolist (p pivots)
        (let ((sub-multiset (remove1 p multiset)))
          (dolist (sub-perm
               (make-k-permutations (- k 1) sub-multiset))
            (push (cons p sub-perm) acc))))
       acc))))

(define (remove1 elt lst)
 (let ((elt-pos (find elt lst)))
   (if elt-pos (pop lst elt-pos))
   lst))

(define (permutations lst)
  (if (= (length lst) 1)
   lst
   (apply append (map (fn (rot) (map (fn (perm) (cons (first rot) perm))
      (permutations (rest rot))))
    (rotations lst)))))

(define (rotations lst)
  (map (fn (x) (rotate lst)) (sequence 1 (length lst))))

(define (inv x y p)
  (if (> x y)
    1
    0))
    
(define (inversions lst)
	(let ((s 0))
  	(for (ex 1 (length lst))
			(if (> (+ ex 1) (length lst))(set 's (+ s 0))
				(for (ey (+ ex 1) (length lst))
					(set 's (+ s (inv (nth (- ex 1) lst) (nth (- ey 1) lst))))
				)))
	)
)

(define (sign lst)
	(pow (- 1) (inversions lst))
)

(define (matpos x y d)
	(add x (mul d (- y 1)))
)

(define (deter matr)
; column-major order
	(let (
				(p1 (permutations (sequence 1 (sqrt (length matr))))) ; list with index permutations
				(n-row (sqrt (length matr)))
				(idx '()) 
				(sig 0) 
				(pos 0)
				(s-tot 1) 
				(tot 0) ) 
					(for (x 1 (length p1))
								(begin        ; we have to linearize the matrix
								                        ; and calculate the index from the row/col
                                        ; indexes. (i,j) = (j-1) * d1 + i
																	                 ; where d1 is the number of rows.
						(set 'idx (nth (- x 1) p1))   ; x-th permutation list
;(println "perm " idx)
						(set 'sig (sign idx))                ; signature of this permutation
;(println "sign " sig)
						(set 's-tot 1)
						(for (i-idx 1 n-row)    ; work on the permutation elements
							(set 'pos (matpos i-idx (nth (- i-idx 1) idx) n-row )) ; this is the linearposition
                                                                     ; in the matrix. Column-major order
;(println "pos " pos)
							(set 's-tot (mul s-tot (nth (- pos 1) matr)))
;(println "subt " s-tot)
						)
						(set 'tot (add tot (mul sig s-tot)))
;(println "tot " tot)
					)))
)

(define (cdeter matr)
; column-major order
	(let (
				(p1 (permutations (sequence 1 (sqrt (length matr))))) ; list with index permutations
				(n-row (sqrt (length matr)))
				(idx '()) 
				(sig 0) 
				(pos 0)
				(s-tot '(1 0)) 
				(tot '(0 0)) ) 
					(for (x 1 (length p1))
								(begin        ; we have to linearize the matrix
								                        ; and calculate the index from the row/col
                                        ; indexes. (i,j) = (j-1) * d1 + i
																	                 ; where d1 is the number of rows.
						(set 'idx (nth (- x 1) p1))   ; x-th permutation list
(println "perm " idx)
						(set 'sig (sign idx))                ; signature of this permutation
(println "sign " sig)
						(set 's-tot '(1 0))
						(for (i-idx 1 n-row)    ; work on the permutation elements
							(set 'pos (matpos i-idx (nth (- i-idx 1) idx) n-row )) ; this is the linearposition
                                                                     ; in the matrix. Column-major order
(println "pos " pos)
							(set 's-tot (cmul s-tot (nth (- pos 1) matr)))
(println "subt " s-tot)
						)
						(set 'tot (cadd tot (cmul sig s-tot)))
(println "tot " tot)
					)))
)						
                                
;
; -- test code
;
;(permutations '(1 2 3))
;(map sign (permutations '(1 2 3)))
;(permutations '(1 2))
;(map sign (permutations '(1 2)))
;(make-k-permutations 3 '(1 2 3))
;(inversions '(1 3 2))
;(inversions '(1 2 3))
;(inversions '(1 2 3 4))
;(inversions '(2 1 4 3))
;(sign '(2 1 3 4))
;(inversions '(5 4 2 3 1))
;(deter '(2 2 2 2))
;(deter '(3 1 3 2 1 1 1 2 2))
;(sign '(1 2 3))
;(inversions '(2 3 1))
;(sign '(2 3 1))
;(inversions '(3 1 2))
;(sign '(3 1 2))